@@ -1,3 +1,4 @@
+# frozen_string_literal: true
 
 module Fugit
 
@@ -12,9 +13,9 @@ module Fugit
       '@daily' => '0 0 * * *',
       '@midnight' => '0 0 * * *',
       '@noon' => '0 12 * * *',
-      '@hourly' => '0 * * * *' }
+      '@hourly' => '0 * * * *' }.freeze
     MAXDAYS = [
-      nil, 31, 29, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 ]
+      nil, 31, 29, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 ].freeze
 
     attr_reader(
       :original, :zone)
@@ -78,23 +79,37 @@ module Fugit
       %w[ year month day wday hour min sec wday_in_month rweek rday ]
         .collect(&:to_sym).each { |k| define_method(k) { @t.send(k) } }
 
-      def inc(i)
-        @t = @t + i
-        self
-      end
+      def inc(i); @t = @t + i; self; end
       def dec(i); inc(-i); end
 
       def inc_month
+
         y = @t.year
         m = @t.month + 1
         if m == 13; m = 1; y += 1; end
+
         @t = ::EtOrbi.make(y, m, @t.zone)
+
         self
       end
 
       def inc_day
+
         inc((24 - @t.hour) * 3600 - @t.min * 60 - @t.sec)
+
+        return if @t.hour == 0
+
+        if @t.hour < 12
+          begin
+            @t = ::EtOrbi.make(@t.year, @t.month, @t.day, @t.zone)
+          rescue ::TZInfo::PeriodNotFound
+            inc((24 - @t.hour) * 3600)
+          end
+        else
+          inc((24 - @t.hour) * 3600)
+        end
       end
+
       def inc_hour
         inc((60 - @t.min) * 60 - @t.sec)
       end
@@ -111,12 +126,7 @@ module Fugit
       end
 
       def dec_month
-
-        #dec(@t.day * 24 * 3600 + @t.hour * 3600 + @t.min * 60 + @t.sec + 1)
-          #
-          # gh-18, so that '0 9 29 feb *' doesn't get skipped (over and over)
-          #
-        dec(@t.day * 24 * 3600 + 1)
+        dec((@t.day - 1) * DAY_S + @t.hour * 3600 + @t.min * 60 + @t.sec + 1)
       end
 
       def dec_day
@@ -155,7 +165,7 @@ module Fugit
 
     def weekday_modulo_match?(nt, mod)
 
-      nt.rweek % mod[0] == mod[1]
+      (nt.rweek + mod[1]) % mod[0] == 0
     end
 
     def weekday_match?(nt)
@@ -236,6 +246,8 @@ module Fugit
         # the translation occurs in the timezone of
         # this Fugit::Cron instance
 
+      zfrom = t.time.strftime('%z|%Z')
+
       loop do
 
         fail RuntimeError.new(
@@ -251,8 +263,14 @@ module Fugit
         min_match?(t) || (t.inc_min; next)
         sec_match?(t) || (t.inc_sec; next)
 
-        st = t.time.strftime('%F|%T')
-        (from, sfrom, ifrom = t.time, st, t.to_i; next) if st == sfrom
+        tt = t.time
+        st = tt.strftime('%F|%T')
+        zt = tt.strftime('%z|%Z')
+          #
+        if st == sfrom && zt != zfrom
+          from, sfrom, zfrom, ifrom = tt, st, zt, t.to_i
+          next
+        end
           #
           # when transitioning out of DST, this prevents #next_time from
           # yielding the same literal time twice in a row, see gh-6
@@ -328,7 +346,7 @@ module Fugit
       [ :seconds, 1, 60 ],
       [ :minutes, 60, 60 ],
       [ :hours, 3600, 24 ],
-      [ :days, 24 * 3600, 365 ] ]
+      [ :days, DAY_S, 365 ] ].freeze
 
     def rough_frequency
 
@@ -370,7 +388,7 @@ module Fugit
 
         @delta_min = deltas.min; @delta_max = deltas.max
         @occurrences = deltas.size
-        @span_years = span / (365 * 24 * 3600)
+        @span_years = span / YEAR_S
         @yearly_occurrences = @occurrences.to_f / @span_years
       end
 
@@ -482,12 +500,14 @@ module Fugit
 
       sla = nil if sla == 1 # don't get fooled by /1
 
+      edn = max if sla && edn.nil?
+
       return [ nil ] if sta.nil? && edn.nil? && sla.nil?
       return [ sta ] if sta && edn.nil?
 
       sla = 1 if sla == nil
       sta = min if sta == nil
-      edn = max if edn == nil
+      edn = max if edn == nil || edn < 0 && sta > 0
 
       range(min, max, sta, edn, sla)
     end
@@ -499,12 +519,10 @@ module Fugit
         { min: min, max: max, sta: sta, edn: edn, sla: sla }.inspect
       ) if (sta < 0 && edn > 0) || (edn < 0 && sta > 0)
 
-#p({ min: min, max: max, sta: sta, edn: edn, sla: sla })
       a = []
 
       omin, omax = min, max
       min, max = -max, -1 if sta < 0
-#p({ min: min, max: max })
 
       cur = sta
 
@@ -604,10 +622,18 @@ module Fugit
 
     module Parser include Raabro
 
-      WEEKDAYS = %w[ sunday monday tuesday wednesday thursday friday saturday ]
-      WEEKDS = WEEKDAYS.collect { |d| d[0, 3] }
+      WEEKDAYS =
+        %w[ sunday monday tuesday wednesday thursday friday saturday ].freeze
 
-      MONTHS = %w[ - jan feb mar apr may jun jul aug sep oct nov dec ]
+      WEEKDS =
+        WEEKDAYS.collect { |d| d[0, 3] }.freeze
+      DOW_REX =
+        /([0-7]|#{WEEKDS.join('|')})/i.freeze
+
+      MONTHS =
+        %w[ - jan feb mar apr may jun jul aug sep oct nov dec ].freeze
+      MONTH_REX =
+        /(1[0-2]|0?[1-9]|#{MONTHS[1..-1].join('|')})/i.freeze
 
       # piece parsers bottom to top
 
@@ -621,8 +647,8 @@ module Fugit
       def mos(i); rex(:mos, i, /[0-5]?\d/); end # min or sec
       def hou(i); rex(:hou, i, /(2[0-4]|[01]?[0-9])/); end
       def dom(i); rex(:dom, i, /(-?(3[01]|[12][0-9]|0?[1-9])|last|l)/i); end
-      def mon(i); rex(:mon, i, /(1[0-2]|0?[1-9]|#{MONTHS[1..-1].join('|')})/i); end
-      def dow(i); rex(:dow, i, /([0-7]|#{WEEKDS.join('|')})/i); end
+      def mon(i); rex(:mon, i, MONTH_REX); end
+      def dow(i); rex(:dow, i, DOW_REX); end
 
       def dow_hash(i); rex(:hash, i, /#(-?[1-5]|last|l)/i); end
 
@@ -765,8 +791,12 @@ module Fugit
 
       def rewrite_cron(t)
 
-        hcron = t
+        st = t
           .sublookup(nil) # go to :ccron or :scron
+
+        return nil unless st
+
+        hcron = st
           .subgather(nil) # list min, hou, mon, ...
           .inject({}) { |h, tt|
             h[tt.name] = tt.name == :tz ? rewrite_tz(tt) : rewrite_entry(tt)
