@@ -107,6 +107,20 @@ describe Fugit::Cron do
       '2019-04-21 10:00:00', '2019-04-11 09:00:00', 'Europe/Berlin' ],
     [ '0 10 * * sun%2+1',
       '2019-04-14 10:00:00', '2019-04-11 09:00:00', 'Europe/Berlin' ],
+
+    # gh-52
+
+    [ '59 23 * * 2', '2021-02-02 23:59:00', '2021-02-02 00:00:00' ],
+    [ '59 23 * * 2', '2021-02-02 23:59:00', '2021-02-02 00:00:00', 'UTC' ],
+    #[ '59 23 * * 2', '2021-02-02 23:59:00', '2021-02-02 00:00:00', 'utc' ],
+
+    [ '59 18 * * 2#2', '2021-02-09 18:59:00', '2021-02-09 17:41:10' ],
+    [ '59 18 * * 2#2', '2021-02-09 18:59:00', '2021-02-09 17:41:10', 'UTC' ],
+    #[ '59 18 * * 2#1', '2021-02-09 18:59:00', '2021-02-09 17:41:10', 'utc' ],
+
+    [ '15/30 * * * *', '2021-02-09 19:15:00', '2021-02-09 19:00:00' ],
+    [ '15/30 * * * *', '2021-02-09 19:45:00', '2021-02-09 19:30:00' ],
+    [ '15-40/30 * * * *', '2021-02-09 20:15:00', '2021-02-09 19:30:00' ],
   ]
 
   describe '#next_time' do
@@ -205,6 +219,126 @@ describe Fugit::Cron do
         end
       end
 
+      it 'correctly increments into DST (gh-53 a)' do
+
+        in_zone 'Europe/Zurich' do
+
+          c = Fugit::Nat.parse('every monday at midnight')
+          t = EtOrbi::EoTime.parse('2021-03-14 12:00:00')
+
+          points =
+            4.times.collect do
+              t = c.next_time(t)
+              tu = t.dup.utc
+              "#{t.strftime('%F_%H:%M_%Z')}__#{tu.strftime('%F_%H:%M_%Z')}"
+            end
+
+          expect(points.join("\n")).to eq(%w[
+            2021-03-15_00:00_CET__2021-03-14_23:00_UTC
+            2021-03-22_00:00_CET__2021-03-21_23:00_UTC
+            2021-03-29_00:00_CEST__2021-03-28_22:00_UTC
+            2021-04-05_00:00_CEST__2021-04-04_22:00_UTC
+          ].join("\n"))
+        end
+      end
+
+      it 'correctly increments into DST (gh-53 b)' do
+
+        in_zone 'Europe/Zurich' do
+
+          #c = Fugit::Nat.parse('every monday at midnight')
+          c = Fugit::Cron.parse('0 0 * * 1')
+          t = EtOrbi::EoTime.parse('2021-03-14 12:00:00')
+
+          points =
+            4.times.collect do
+              t = c.next_time(t)
+              tu = t.dup.utc
+              "#{t.strftime('%F_%H:%M_%Z')}__#{tu.strftime('%F_%H:%M_%Z')}"
+            end
+
+          expect(points.join("\n")).to eq(%w[
+            2021-03-15_00:00_CET__2021-03-14_23:00_UTC
+            2021-03-22_00:00_CET__2021-03-21_23:00_UTC
+            2021-03-29_00:00_CEST__2021-03-28_22:00_UTC
+            2021-04-05_00:00_CEST__2021-04-04_22:00_UTC
+          ].join("\n"))
+        end
+      end
+
+      it 'correctly increments into DST (gh-53 c)' do
+
+        in_zone 'Europe/Zurich' do
+
+          #c = Fugit::Nat.parse('every monday at midnight')
+          c = Fugit::Cron.parse('0 0 * * 2')
+          t = EtOrbi::EoTime.parse('2021-03-14 12:00:00')
+
+          points =
+            4.times.collect do
+              t = c.next_time(t)
+              tu = t.dup.utc
+              "#{t.strftime('%F_%H:%M_%Z')}__#{tu.strftime('%F_%H:%M_%Z')}"
+            end
+
+          expect(points.join("\n")).to eq(%w[
+            2021-03-16_00:00_CET__2021-03-15_23:00_UTC
+            2021-03-23_00:00_CET__2021-03-22_23:00_UTC
+            2021-03-30_00:00_CEST__2021-03-29_22:00_UTC
+            2021-04-06_00:00_CEST__2021-04-05_22:00_UTC
+          ].join("\n"))
+        end
+      end
+
+      it 'correctly increments into DST (gh-53 d)' do
+
+        in_zone 'Europe/Zurich' do
+
+          #c = Fugit::Nat.parse('every monday at midnight')
+          #c = Fugit::Cron.parse('0 0 * * 2')
+          c = Fugit::Nat.parse('every tuesday at 00:00')
+          t = EtOrbi::EoTime.parse('2021-03-14 12:00:00')
+
+          points =
+            4.times.collect do
+              t = c.next_time(t)
+              tu = t.dup.utc
+              "#{t.strftime('%F_%H:%M_%Z')}__#{tu.strftime('%F_%H:%M_%Z')}"
+            end
+
+          expect(points.join("\n")).to eq(%w[
+            2021-03-16_00:00_CET__2021-03-15_23:00_UTC
+            2021-03-23_00:00_CET__2021-03-22_23:00_UTC
+            2021-03-30_00:00_CEST__2021-03-29_22:00_UTC
+            2021-04-06_00:00_CEST__2021-04-05_22:00_UTC
+          ].join("\n"))
+        end
+      end
+
+      it 'correctly increments out of DST (gh-53 e)' do
+
+        in_zone 'Europe/Zurich' do
+
+          #c = Fugit::Nat.parse('every monday at midnight')
+          c = Fugit::Cron.parse('0 0 * * 2')
+          t = EtOrbi::EoTime.parse('2021-10-18 12:00:00')
+
+          points =
+            4.times.collect do
+              t = c.next_time(t)
+              tu = t.dup.utc
+              "#{t.strftime('%F_%H:%M_%Z')}__#{tu.strftime('%F_%H:%M_%Z')}"
+            end
+
+          expect(points.join("\n")).to eq(%w[
+            2021-10-19_00:00_CEST__2021-10-18_22:00_UTC
+            2021-10-26_00:00_CEST__2021-10-25_22:00_UTC
+            2021-11-02_00:00_CET__2021-11-01_23:00_UTC
+            2021-11-09_00:00_CET__2021-11-08_23:00_UTC
+          ].join("\n"))
+        end
+      end
+
       it 'correctly increments out of DST' do
 
         in_zone 'America/Los_Angeles' do
@@ -259,6 +393,154 @@ describe Fugit::Cron do
           ).to eq(365)
         end
       end
+
+      it 'correctly increments into DST (America/Santiago) gh-60' do
+
+        in_zone 'America/Santiago' do
+
+          c = Fugit.parse('0 8 15 * *')
+          t = EtOrbi::EoTime.parse('2021-06-18 01:00:00')
+
+          points =
+            6.times
+              .collect {
+                t = c.next_time(t)
+                t.to_zs + ' // ' + t.to_t.to_s }
+              .join("\n")
+
+          expect(points).to eq(%{
+            2021-07-15 08:00:00 America/Santiago // 2021-07-15 08:00:00 -0400
+            2021-08-15 08:00:00 America/Santiago // 2021-08-15 08:00:00 -0400
+            2021-09-15 08:00:00 America/Santiago // 2021-09-15 08:00:00 -0300
+            2021-10-15 08:00:00 America/Santiago // 2021-10-15 08:00:00 -0300
+            2021-11-15 08:00:00 America/Santiago // 2021-11-15 08:00:00 -0300
+            2021-12-15 08:00:00 America/Santiago // 2021-12-15 08:00:00 -0300
+          }.strip.split("\n").collect(&:strip).join("\n"))
+
+          expect(
+            c.brute_frequency(2021).occurrences
+          ).to eq(12)
+        end
+      end
+
+      it 'correctly increments into DST (America/Santiago) Time.zone gh-62' do
+
+        in_active_support_zone 'America/Santiago' do
+
+          c = Fugit.parse('0 8 15 * *')
+          t = EtOrbi::EoTime.parse('2021-06-18 01:00:00')
+
+          points =
+            6.times
+              .collect {
+                t = c.next_time(t)
+                t.to_zs + ' // ' + t.to_t.dup.utc.to_s }
+              .join("\n")
+
+          expect(points).to eq(%{
+            2021-07-15 08:00:00 America/Santiago // 2021-07-15 12:00:00 UTC
+            2021-08-15 08:00:00 America/Santiago // 2021-08-15 12:00:00 UTC
+            2021-09-15 08:00:00 America/Santiago // 2021-09-15 11:00:00 UTC
+            2021-10-15 08:00:00 America/Santiago // 2021-10-15 11:00:00 UTC
+            2021-11-15 08:00:00 America/Santiago // 2021-11-15 11:00:00 UTC
+            2021-12-15 08:00:00 America/Santiago // 2021-12-15 11:00:00 UTC
+          }.strip.split("\n").collect(&:strip).join("\n"))
+
+          expect(
+            c.brute_frequency(2021).occurrences
+          ).to eq(12)
+        end
+      end
+
+      it 'correctly increments into DST (America/Santiago) Time.zone gh-62 hourly' do
+
+        in_active_support_zone 'America/Santiago' do
+
+          c = Fugit.parse('0 * * * *')
+          t = EtOrbi::EoTime.parse('2021-09-04 21:00:00')
+
+          points =
+            6.times
+              .collect {
+                t = c.next_time(t)
+                t.to_zs + ' // ' + t.to_t.dup.utc.to_s }
+              .join("\n")
+
+          expect(points).to eq(
+            %{
+              2021-09-04 22:00:00 America/Santiago // 2021-09-05 02:00:00 UTC
+              2021-09-04 23:00:00 America/Santiago // 2021-09-05 03:00:00 UTC
+              2021-09-05 01:00:00 America/Santiago // 2021-09-05 04:00:00 UTC
+              2021-09-05 02:00:00 America/Santiago // 2021-09-05 05:00:00 UTC
+              2021-09-05 03:00:00 America/Santiago // 2021-09-05 06:00:00 UTC
+              2021-09-05 04:00:00 America/Santiago // 2021-09-05 07:00:00 UTC
+            }
+              .strip.split("\n").map(&:strip)
+              .reject { |l| l[0, 1] == '#' }.join("\n"))
+
+          expect(
+            c.brute_frequency(2021).occurrences
+          ).to eq(8759)
+        end
+      end
+
+      it 'correctly increments out of DST (America/Santiago) gh-60' do
+
+        in_zone 'America/Santiago' do
+
+          c = Fugit.parse('0 8 15 * *')
+          t = EtOrbi::EoTime.parse('2021-01-12 01:00:00')
+
+          points =
+            6.times
+              .collect {
+                t = c.next_time(t)
+                t.to_zs + ' // ' + t.to_t.to_s }
+              .join("\n")
+
+          expect(points).to eq(%{
+            2021-01-15 08:00:00 America/Santiago // 2021-01-15 08:00:00 -0300
+            2021-02-15 08:00:00 America/Santiago // 2021-02-15 08:00:00 -0300
+            2021-03-15 08:00:00 America/Santiago // 2021-03-15 08:00:00 -0300
+            2021-04-15 08:00:00 America/Santiago // 2021-04-15 08:00:00 -0400
+            2021-05-15 08:00:00 America/Santiago // 2021-05-15 08:00:00 -0400
+            2021-06-15 08:00:00 America/Santiago // 2021-06-15 08:00:00 -0400
+          }.strip.split("\n").collect(&:strip).join("\n"))
+
+          expect(
+            c.brute_frequency(2021).occurrences
+          ).to eq(12)
+        end
+      end
+
+      it 'correctly increments out of DST (America/Santiago) Time.zone gh-62' do
+
+        in_active_support_zone 'America/Santiago' do
+
+          c = Fugit.parse('0 8 15 * *')
+          t = EtOrbi::EoTime.parse('2021-01-12 01:00:00')
+
+          points =
+            6.times
+              .collect {
+                t = c.next_time(t)
+                t.to_zs + ' // ' + t.to_t.dup.utc.to_s }
+              .join("\n")
+
+          expect(points).to eq(%{
+            2021-01-15 08:00:00 America/Santiago // 2021-01-15 11:00:00 UTC
+            2021-02-15 08:00:00 America/Santiago // 2021-02-15 11:00:00 UTC
+            2021-03-15 08:00:00 America/Santiago // 2021-03-15 11:00:00 UTC
+            2021-04-15 08:00:00 America/Santiago // 2021-04-15 12:00:00 UTC
+            2021-05-15 08:00:00 America/Santiago // 2021-05-15 12:00:00 UTC
+            2021-06-15 08:00:00 America/Santiago // 2021-06-15 12:00:00 UTC
+          }.strip.split("\n").collect(&:strip).join("\n"))
+
+          expect(
+            c.brute_frequency(2021).occurrences
+          ).to eq(12)
+        end
+      end
     end
 
     it 'returns a plain second' do
@@ -346,34 +628,91 @@ describe Fugit::Cron do
       before :each do
 
         require_chronic
-
-        class ::Time
-          class << self
-            def zone; @zone; end
-          end
-        end
       end
 
       after :each do
 
         unrequire_chronic
-
-        class ::Time
-          class << self
-            undef_method :zone
-          end
-        end
       end
 
       it "doesn't stall or loop ad infinitum" do
 
-        Time._zone = 'UTC'
+        in_active_support_zone('UTC') do
 
-        cron = Fugit.do_parse_cron('0 0 1 1 *')
+          cron = Fugit.do_parse_cron('0 0 1 1 *')
 
-        expect {
-          cron.next_time
-        }.not_to raise_error
+          expect {
+            cron.next_time
+          }.not_to raise_error
+        end
+      end
+    end
+
+    context 'New York skip (gh-43)' do
+
+      it "doesn't skip" do
+
+        cron = Fugit.parse('0 8-19/4 * * *')
+
+        st = Time.parse('2020-09-11 12:00:00')
+
+        nt = cron.next_time(st)
+#p nt
+#p nt.to_s
+#p nt.to_local_time
+#p nt.utc.to_s
+
+        expect(nt.to_s).to match(/ 16:00:00 /)
+      end
+
+      it "doesn't skip (TZ UTC)" do
+
+        in_zone('UTC') do
+
+          cron = Fugit.parse('0 8-19/4 * * *')
+
+          st = Time.parse('2020-09-11 12:00:00')
+
+          nt = cron.next_time(st)
+#p nt
+#p nt.to_s
+#p nt.to_local_time
+#p nt.utc.to_s
+
+          expect(nt.utc.to_s).to match(/ 16:00:00 /)
+        end
+      end
+
+      it "doesn't skip (ActiveSupport TZ America/New_York)" do
+
+        in_active_support_zone('America/New_York') do
+#EtOrbi._make_info
+#p EtOrbi.determine_local_tzone
+
+          cron = Fugit.parse('0 8-19/4 * * *')
+
+          st = Time.parse('2020-09-11 12:00:00 UTC')
+#p st
+
+          nt = cron.next_time(st)
+#p nt
+#p nt.to_s
+#p nt.to_local_time
+#p nt.utc.to_s
+
+          expect(nt.utc.to_s).to match(/ 16:00:00 /)
+        end
+      end
+
+      it 'does not break on "* * * * 1%2+2" (gh-47)' do
+
+        cron0 = Fugit.parse('0 8 * * 1%2+2')
+        cron1 = Fugit.parse('0 8 * * 1%2')
+
+        expect(cron0.next_time('2021-04-21 07:00:00').to_s
+          ).to match(/^2021-05-03 08:00:00 /)
+        expect(cron0.next_time('2021-04-21 07:00:00').to_s
+          ).to eq(cron1.next_time('2021-04-21 07:00:00').to_s)
       end
     end
   end
@@ -447,6 +786,7 @@ describe Fugit::Cron do
     [ '0 0 * * mon#2,tue', '2017-01-09', '2017-01-09 12:00' ],
     [ '0 0 * * mon#2,tue', '2017-01-03', '2017-01-04' ],
 
+    [ '0 20 31 oct *', '2019-10-31 20:00', '2019-11-01' ], # gh-51
     [ '0 9 29 feb *', '2016-02-29 09:00', '2019-03-23' ], # gh-18
 
     [ '59 6 1-7 * 2', '2020-03-10 06:59:00', '2020-03-15 07:47' ],
@@ -762,6 +1102,16 @@ describe Fugit::Cron do
           #
           # gh-35
 
+        [ '0 8-19/4 * * *', '0 8,12,16 * * *' ],
+          #
+          # gh-43
+
+        [ '1 */1 * * *', '1 * * * *' ],
+        [ '1 0/1 * * *', '1 0 * * *' ],
+        [ '1 0-23/1 * * *', "1 #{(0..23).map(&:to_s).join(',')} * * *" ],
+          #
+          # some cronds are different, see https://crontab.guru/#7_0/1_*_*_*
+
       ].each { |c, e|
 
         it "parses #{c}" do
@@ -781,6 +1131,11 @@ describe Fugit::Cron do
           [ '* * L * *', '* * -1 * *' ],
           [ '* * -7-L * *', '* * -7,-6,-5,-4,-3,-2,-1 * *' ],
           [ '* * last * *', '* * -1 * *' ],
+
+          [ '* * 25-l * *', "* * #{(25..31).to_a.map(&:to_s).join(',')} * *" ],
+          [ '* * 25-L * *', "* * #{(25..31).to_a.map(&:to_s).join(',')} * *" ],
+            # not so negative...
+
         ].each { |c, e|
           it("parses #{c}") { expect(Fugit::Cron.parse(c).to_cron_s).to eq(e) }
         }
@@ -865,6 +1220,10 @@ describe Fugit::Cron do
       [
         # min hou dom mon dow
 
+        nil,
+        '',
+        ' ',
+
         '* 25 * * *',
         '* * -32 * *',
 
